<?php
/**
 * Plugin Name: bitpay Paid Memberships Pro
 * Description: درگاه پرداخت بیت پی برای افزونه Paid Memberships Pro
 * Author: Farhan
 * Version: 1.0
 * License: GPL v2.0
 */

//load classes init method
add_action('plugins_loaded', 'load_bitpay_pmpro_class', 11 );
add_action('plugins_loaded', array( 'PMProGateway_bitpay', 'init' ), 12 );

function load_bitpay_pmpro_class()
{
    if( class_exists( 'PMProGateway' ) )
    {
        class PMProGateway_bitpay extends PMProGateway
        {
            function PMProGateway_bitpay( $gateway = NULL )
            {
                $this->gateway = $gateway;
                $this->gateway_environment = pmpro_getOption( "gateway_environment" );

                return $this->gateway;
            }
            
            static function init()
            {
                //make sure Stripe is a gateway option
                add_filter('pmpro_gateways', array( 'PMProGateway_bitpay', 'pmpro_gateways' ) );

                //add fields to payment settings
                add_filter('pmpro_payment_options', array('PMProGateway_bitpay', 'pmpro_payment_options'));
                add_filter('pmpro_payment_option_fields', array('PMProGateway_bitpay', 'pmpro_payment_option_fields'), 10, 2);
                $gateway = pmpro_getOption("gateway");
                if($gateway == "bitpay")
                {
                    add_filter('pmpro_checkout_before_change_membership_level', array('PMProGateway_bitpay', 'pmpro_checkout_before_change_membership_level'), 10, 2);
                    add_filter('pmpro_include_billing_address_fields', '__return_false');
                    add_filter('pmpro_include_payment_information_fields', '__return_false');
                    add_filter('pmpro_required_billing_fields', array('PMProGateway_bitpay', 'pmpro_required_billing_fields'));
                }
                
                add_action('wp_ajax_nopriv_bitpay-ins', array( 'PMProGateway_bitpay', 'pmpro_wp_ajax_bitpay_ins') );
                add_action('wp_ajax_bitpay-ins', array( 'PMProGateway_bitpay', 'pmpro_wp_ajax_bitpay_ins') );
                
            }
            
            /**
             * Make sure bitpay is in the gateways list
             *
             * @since 1.0
             */
            static function pmpro_gateways( $gateways )
            {
                if(empty($gateways['bitpay']))
                    $gateways['bitpay'] = 'بیت پی';
                return $gateways;
            }
            
            /**
             * Get a list of payment options that the bitpay gateway needs/supports.
             *
             * @since 1.0
             */
            static function getGatewayOptions()
            {

                $options = array(
                    'bitpay_api'
                );

                return $options;
            }
            
            /**
             * Set payment options for payment settings page.
             *
             * @since 1.0
             */
            static function pmpro_payment_options( $options )
            {
                //get bitpay options
                $bitpay_options = PMProGateway_bitpay::getGatewayOptions();

                //merge with others.
                $options = array_merge( $bitpay_options, $options );

                return $options;
            }
            
            /**
             * Remove required billing fields
             *		 
             * @since 1.8
             */
            static function pmpro_required_billing_fields($fields)
            {
                unset($fields['bfirstname']);
                unset($fields['blastname']);
                unset($fields['baddress1']);
                unset($fields['bcity']);
                unset($fields['bstate']);
                unset($fields['bzipcode']);
                unset($fields['bphone']);
                unset($fields['bemail']);
                unset($fields['bcountry']);
                unset($fields['CardType']);
                unset($fields['AccountNumber']);
                unset($fields['ExpirationMonth']);
                unset($fields['ExpirationYear']);
                unset($fields['CVV']);
                
                return $fields;
            }
            
            /**
             * Display fields for bitpay options.
             *
             * @since 1.0
             */
            static function pmpro_payment_option_fields( $values, $gateway )
            {
?>
<tr class="pmpro_settings_divider gateway gateway_bitpay" <?php if( $gateway != "bitpay" ) { ?>style="display: none;"<?php } ?>>
    <td colspan="2">
        <?php echo 'تنظیمات بیت پی'; ?>
    </td>
</tr>
<tr class="gateway gateway_bitpay" <?php if($gateway != "bitpay") { ?>style="display: none;"<?php } ?>>
    <th scope="row" valign="top">
        <label for="bitpay_api">کد API جهت اتصال به بیت پی:</label>
    </th>
    <td>
        <input type="text" id="bitpay_api" name="bitpay_api" size="60" value="<?php echo esc_attr( $values['bitpay_api'] ); ?>" />
    </td>
</tr>

<?php
            }
          
            /**
             * Instead of change membership levels, send users to bitpay to pay.
             *
             * @since 1.8
             */
            static function pmpro_checkout_before_change_membership_level($user_id, $morder)
            {
                global $wpdb, $discount_code_id;
                
                //if no order, no need to pay
                if( empty( $morder ) )
                    return;
                
                $morder->user_id = $user_id;				
                $morder->saveOrder();
                
                //save discount code use
                if( ! empty( $discount_code_id ) )
                    $wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" . $user_id . "', '" . $morder->id . "', now())");	

                //$morder->Gateway->sendToTwocheckout($morder);
                global $pmpro_currency;
                
                $gtw_env = pmpro_getOption( 'gateway_environment' );

                if( $gtw_env == '' || $gtw_env == 'sandbox' )
                {
                    $api = 'adxcv-zzadq-polkjsad-opp13opoz-1sdf455aadzmck1244567';
                    $url = 'https://bitpay.ir/payment-test/gateway-send';
                }
                else
                {
                    $api = pmpro_getOption( 'bitpay_api' );
                    $url = 'https://bitpay.ir/payment/gateway-send';
                }
                
                $order_id = $morder->code;
                $redirect = admin_url("admin-ajax.php") . "?action=bitpay-ins&oid=$order_id";
                $redirect = urlencode( $redirect );
                
                global $pmpro_currency;
                
                $amount = intval( $morder->subtotal );
                if( $pmpro_currency == 'IRT' )
                    $amount *= 10;
                
                $ch = curl_init();         
                curl_setopt($ch,CURLOPT_URL, $url );          
                curl_setopt($ch,CURLOPT_POSTFIELDS,"api={$api}&amount={$amount}&redirect={$redirect}");
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
                curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
                $res = curl_exec($ch);
                curl_close($ch);

                switch( $res ) {
                    case '-1' :
                        $morder->status = "cancelled";
                        $morder->notes = 'کد api ارسالی با کد api ثبت شده در بیت پی سازگار نیست.';
                        $morder->saveOrder();
                        die('کد api ارسالی با کد api ثبت شده در بیت پی سازگار نیست.');
                    case '-2' :  
                        $morder->status = "cancelled";
                        $morder->notes = 'مبلغ یک مقدار عددی است و حداقل باید 1000 ریال باشد.';
                        $morder->saveOrder();
                        die('مبلغ یک مقدار عددی است و حداقل باید 1000 ریال باشد.');
                    case '-3' : 
                        $morder->status = "cancelled";
                        $morder->notes = 'آدرس بازگشت از درگاه یک مقدار نال می باشد.';
                        $morder->saveOrder();
                        die('آدرس بازگشت از درگاه یک مقدار نال می باشد.');
                    case '-4' :
                        $morder->status = "cancelled";
                        $morder->notes = 'چنین درگاهی وجود ندارد و یا هنوز در انتظار تایید می باشد!';
                        $morder->saveOrder();
                        die('چنین درگاهی وجود ندارد و یا هنوز در انتظار تایید می باشد!');
                    default :
                        if( $res > 0 && is_numeric( $res ) ){
                            if( $gtw_env == '' || $gtw_env == 'sandbox' ) 
                                $go = "https://bitpay.ir/payment-test/gateway-{$res}";
                            else {
                                $go = "https://bitpay.ir/payment/gateway-{$res}";
                            }
                            
                            header("Location: {$go}");
                            die();
                        }
                        else {
                            $morder->status = "cancelled";
                            $morder->notes = 'خطا در ارسال به بیت پی';
                            $morder->saveOrder();
                            die('خطا در ارسال به بیت پی');
                        }
                }
            }
            
            static function pmpro_wp_ajax_bitpay_ins()
            {
                global $gateway_environment;
                if( ! isset( $_GET['oid'] ) || is_null( $_GET['oid'] ) )
                {
                    die( 'meghdare oid dar dargahe bitpay elzamist' );
                }
                
                $oid = $_GET['oid'];
                
                $morder = null;
                try
                {
                    $morder = new MemberOrder( $oid );
                    $morder->getMembershipLevel();
                    $morder->getUser();
                }
                catch (Exception $exception)
                {
                    die( 'meghdare oid na motabar ast' );
                }
                
                $current_user_id = get_current_user_id();

                if( $current_user_id !== intval( $morder->user_id ) )
                {
                    die( 'in kharid motealegh be shoma nist' );
                }

                $gtw_env = pmpro_getOption( 'gateway_environment' );

                if( $gtw_env == '' || $gtw_env == 'sandbox' )
                {
                    $api = 'adxcv-zzadq-polkjsad-opp13opoz-1sdf455aadzmck1244567';
                    $url = 'https://bitpay.ir/payment-test/gateway-result-second';
                }
                else
                {
                    $api = pmpro_getOption( 'bitpay_api' );
                    $url = 'https://bitpay.ir/payment/gateway-result-second';
                }
                
                if( ! isset( $_POST['trans_id'] ) || ! isset( $_POST['id_get'] ) ){
                    die( 'به نظر می رسد درگاه بیت پی دچار مشکلی شده و یا قواعد اتصال به درگاه تغییر کرده لطفا مسئول سایت را از این موضوع مطلع نمایید.' );
                }
                
                $trans_id = $_POST['trans_id']; 
                $id_get = $_POST['id_get']; 
                
                $ch = curl_init();     
                curl_setopt($ch,CURLOPT_URL,$url);     
                curl_setopt($ch,CURLOPT_POSTFIELDS,"api=$api&id_get=$id_get&trans_id=$trans_id");
                curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
                curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);
                $res = curl_exec($ch);
                curl_close($ch); 
                global $pmpro_msgt, $pmpro_msg;
                switch( $res ) {
                    case '-1' :  
                        $morder->status = "cancelled";
                        $morder->notes = 'کد api ارسالی با کد api ثبت شده در بیت پی سازگار نیست.';
                        $morder->saveOrder();
                        header("Location: " . pmpro_url() );
                        die();
                    case '-2' :  
                        $morder->status = "cancelled";
                        $morder->notes = 'شماره تراکنش ارسالی نامعتبر است.';
                        $morder->saveOrder();
                        header("Location: " . pmpro_url() );
                        die();
                    case '-3' : 
                        $morder->status = "cancelled";
                        $morder->notes = 'id_get ارسالی نامعتبر می باشد.';
                        $morder->saveOrder();
                        header("Location: " . pmpro_url() );
                        die();
                    case '-4' :
                        $morder->status = "cancelled";
                        $morder->notes = 'چنین تراکنشی وجود ندارد و یا قبلا با موفقیت به اتمام رسیده است. همچنین امکان دارد عملیات پرداخت توسط کاربر لغو شده باشد.';
                        $morder->saveOrder();
                        header("Location: " . pmpro_url() );
                        die();
                    case '1' :
                        if( self::do_level_up( $morder, $trans_id ) )
                        {
                           header( 'Location:' . pmpro_url( "confirmation", "?level=" . $morder->membership_level->id ) );
                        }
                    default:
                        $morder->status = "cancelled";
                        $morder->notes = 'خطایی دور از انتظار در هنگام برگشت کاربر از خرید رخ داد.';
                        $morder->saveOrder();
                        header("Location: " . pmpro_url() );
                        die();
                }   
            }
            
            static function do_level_up( &$morder, $txn_id )
            {
                global $wpdb;
                //filter for level
                $morder->membership_level = apply_filters("pmpro_inshandler_level", $morder->membership_level, $morder->user_id);
                
                //fix expiration date		
                if(!empty($morder->membership_level->expiration_number))
                {
                    $enddate = "'" . date("Y-m-d", strtotime("+ " . $morder->membership_level->expiration_number . " " . $morder->membership_level->expiration_period, current_time("timestamp"))) . "'";
                }
                else
                {
                    $enddate = "NULL";
                }
                
                //get discount code
                $morder->getDiscountCode();
                if(!empty($morder->discount_code))
                {		
                    //update membership level
                    $morder->getMembershipLevel(true);
                    $discount_code_id = $morder->discount_code->id;
                }
                else
                    $discount_code_id = "";
                
                //set the start date to current_time('mysql') but allow filters
                $startdate = apply_filters("pmpro_checkout_start_date", "'" . current_time('mysql') . "'", $morder->user_id, $morder->membership_level);
                
                //custom level to change user to
                $custom_level = array(
                    'user_id' => $morder->user_id,
                    'membership_id' => $morder->membership_level->id,
                    'code_id' => $discount_code_id,
                    'initial_payment' => $morder->membership_level->initial_payment,
                    'billing_amount' => $morder->membership_level->billing_amount,
                    'cycle_number' => $morder->membership_level->cycle_number,
                    'cycle_period' => $morder->membership_level->cycle_period,
                    'billing_limit' => $morder->membership_level->billing_limit,
                    'trial_amount' => $morder->membership_level->trial_amount,
                    'trial_limit' => $morder->membership_level->trial_limit,
                    'startdate' => $startdate,
                    'enddate' => $enddate);

                global $pmpro_error;
                if(!empty($pmpro_error))
                {
                    echo $pmpro_error;
                    inslog($pmpro_error);				
                }
                
                if( pmpro_changeMembershipLevel($custom_level, $morder->user_id) !== false ) {
                    //update order status and transaction ids					
                    $morder->status = "success";
                    $morder->payment_transaction_id = $txn_id;
                    //if( $recurring )
                    //    $morder->subscription_transaction_id = $txn_id;
                    //else
                    $morder->subscription_transaction_id = '';
                    $morder->saveOrder();
                    
                    //add discount code use
                    if(!empty($discount_code) && !empty($use_discount_code))
                    {
                        $wpdb->query("INSERT INTO $wpdb->pmpro_discount_codes_uses (code_id, user_id, order_id, timestamp) VALUES('" . $discount_code_id . "', '" . $morder->user_id . "', '" . $morder->id . "', '" . current_time('mysql') . "')");
                    }									
                    
                    //save first and last name fields
                    if(!empty($_POST['first_name']))
                    {
                        $old_firstname = get_user_meta($morder->user_id, "first_name", true);
                        if(!empty($old_firstname))
                            update_user_meta($morder->user_id, "first_name", $_POST['first_name']);
                    }
                    if(!empty($_POST['last_name']))
                    {
                        $old_lastname = get_user_meta($morder->user_id, "last_name", true);
                        if(!empty($old_lastname))
                            update_user_meta($morder->user_id, "last_name", $_POST['last_name']);
                    }
                    
                    //hook
                    do_action("pmpro_after_checkout", $morder->user_id);						
                    
                    //setup some values for the emails
                    if(!empty($morder))
                        $invoice = new MemberOrder($morder->id);						
                    else
                        $invoice = NULL;
                    
                    //inslog("CHANGEMEMBERSHIPLEVEL: ORDER: " . var_export($morder, true) . "\n---\n");
                    
                    $user = get_userdata( intval( $morder->user_id ) );					
                    if(empty($user))
                        return false;

                    $user->membership_level = $morder->membership_level;		//make sure they have the right level info
                    
                    //send email to member
                    $pmproemail = new PMProEmail();				
                    $pmproemail->sendCheckoutEmail($user, $invoice);
                    
                    //send email to admin
                    $pmproemail = new PMProEmail();
                    $pmproemail->sendCheckoutAdminEmail($user, $invoice);

                    return true;
                }
                else
                    return false;
            }
        }
    }
}